
PRO chara_obs_plan3, tele=tele, src_filename=fname, optimize=optimize, obst=obst,npop=npop,$
		    date=day, mon=mon, year=year,daylightsaving=daylightsav, sorting=sorting, $
		    sunset=sunset, sunrise=sunrise, starttime=starttime, endtime=endtime, $
		    max_slot=max_slot, name_slot=name_slot, cvtime=cvtime, verbose=verbose,$
                    path=path, outputid=id, popfix=popfix, gui_id=gid, $
                    target_name=target_name, target_id = target_id

; Purpose : Find the best POP setup and give a time plan for CHARA observation for given telescopes, given 
;		date, and given sources.
;	    The results will be in the output file.
; 
; Input   : tele,  The telescopes to be used, e.g. tele=[0,1,4,5] for S1 S2 W1 W2
;			'S1', 'S2', 'E1', 'E2', 'W1', 'W2' 
;	                  0     1     2     3     4     5
;
;	    src_filename, the file name of source list. e.g. src.txt
;			  The format should be the same as 'src.txt'
;		
;			  e.g.  SOURCE      RA            Dec.       Weight  K       Comment
;                               --------------------------------------------------------------------
;	                        Regulus 10 08 22.3107 +11 58 01.945  1.0    1.4     Elongated
;     	          	        Del Cep 22 29 10.2663 +58 24 54.715  1.0    4.1     Cepheid
;       	                Alp Cep 21 18 34.7715 +62 35 08.061  1.0    2.4     Elongated
;               	        Alp Oph 17 34 56.0705 +12 33 36.125  1.0    2.1     Elongated
;                       	... ...
;
;
;               POPFIX = e.g., [1, 2, 3,4] for 4_Tele
;                        Set this parameter to fix the choice of pops. '0' means no choice, 
;                        e.g., [0,0,0,3] means fix the last pop to be 3 and search for the other pops.
;                        Default = [0,0,0,0]
;
;	    optimize,  	= 1 : maximize the number of available sources and then weighted obs. time of observable sources.
;		  	= 2 : maximize the obs. time of sources based on their weights.
;			Weights are given in the sourse list file.
;		 	Please give large weights to sources that have high priority, e.g. 10 
;
;	    npop,  the number of pop setups that will be taken after the optimization.
;                Default is 3, i.e., for each telescope group, the best 3 pops that satisfy the criteria will be chosen.
;
;	    obst, Obs. time threshold for avaliable sources, in HOUR
;		  i.e. sources that have observable time >= obst are considered as avaliable.
;		  Default is 0.5 hour
;    	    
;  	    day, mon, year, The observation time, used to calculate the sunset and sunrise cutoff. Integers
;
;	    starttime, Observation starts STARTTIME hours after sunset, default = 1.0 hour
;  
;           endtime,   Observation ends ENDTIME hours beforee sunrise, default = 0.0 hour.
;
;	    Sorting = 1 : Sources will be sorted by their observable duration. default =1
;		    = 2 : SOurces will be sorted first by their observation starting time.
;                                                 then by Ending time
;
;	    /daylightsav, set this keyword if using day light saving time
;
; 	    /verbose, set this keyword to get details of the output
;
;           Path, the path of the archive data.
;
;           namefix :  The postfix for the output POPs. This psotfix will make sure the output is consistent with the
;                      REAL POP names, instead of the idl version (i.e., start from 0).
;
;
; Output   : 	max_slot,   the observable time duration of the sources. in HOURs
;            	name_slot,  names of the best POP setup
;	 	cvtime,     the local observation time (start -> end) of the sources.
;		sunset, sunrise,  Local sunset and sunrise time
;             
;
; Usage    : Full Version:
;		 chara_obs_plan, tele=[0,1,4,5], src_filename='src.txt', opti=1, obst =0.5, npop=3, /verbose, $
;			date=22, mon=8, year=2005,/dayli, max_slot=max_slot, name_slot=name_slot, cvtime=cvtime 
;	
;	     or,  Short Version:
;		 chara_obs_plan, tele=[1,4,5], src_filename='src.txt', opti=1, date=22, mon=8, year=2005, /dayli
;
; Note    : The output file will be in 'current' directory where you run IDL.
;
; MZ, 08/2005
; MZ, 05/18/2006, modified to default path and input src list format.
;     05/21/2006, fix bug in the output name of POPs      
;     06/14/2007, Adapted from chara_obs_plan and 
;                 modified for observation plan GUI
;     11/13/2007  Modified the DaylightSaving time. Now change it automaticly but not exact!!!            
;      

If ( mon+(day/30.) ge 3.4) and ( mon+ day/30. le 11.2 ) then daylightsav = 1 else daylightsav=0

If n_elements(sorting) eq 0 then sorting =1
If n_elements(path) eq 0 then begin

path ='poprange/'

endif

n_tele = n_elements(tele)

case n_tele of  
            2: namefix=11
            3: namefix=111
            4: namefix=1111
            5: namefix=11111
            6: namefix=111111
endcase   

telename = ['S1', 'S2', 'E1', 'E2', 'W1', 'W2' ]
;	       0     1     2     3     4     5

tname=''	
for n=0, n_tele-1 do tname = tname+telename[tele[n]]

if n_elements(id) eq 0 then begin
       output = 'src_time_'+tname+'.txt'
       print, 'No output file is given. Using default : ', output
endif else output = id

openw, lun, output, /get_lun

printf, lun, 'Observation plan for '+strtrim(string(mon),1)+'/' +strtrim(string(day),1)+'/'+ strtrim(string(year),1) + ' CHARA Run'
printf, lun, '' 
printf, lun, 'Telescope : ', telename[tele]


  ; read source Ra and Dec date from file and call src_plan to get the time plan.

read_src_chara_plan, file=fname, src_pos=src, weight=wei, src_name=srcname, comment=comment, path=path

src_plan2, src_list= src, weight= wei^2.0, optimize=optimize, obstime=obst, npop=npop, tele=tele, starttime=starttime,$
                        date=day, mon=mon, year=year,daylightsaving=daylightsav, sunset=sunset, sunrise=sunrise,$
                        max_slot=max_slot, name_slot=name_slot, cvtime=cvtime, /verbose, endtime=endtime, path=path,$
                        namefix=namefix, popfix=popfix, gui_id=gid


  ; Output...

if keyword_set(verbose) eq 1 then begin
	print, '--> Observation time for the sources are :'
	print,''
;	head= replicate('start   End    ', npop)
        print, '                                    Slot 1         Slot 2 '  
        print, '                           Source   Start  End   Start  End   Duration   Weight      V      H       K      Comment'

	For m = 0, npop-1 do begin
		print, ' POP: ', name_slot[m]+namefix
              For n= 0, n_elements(src[0,*])-1 do begin

                  if sorting eq 1 then n2 = reverse(sort(max_slot[*,m]))
                  if sorting eq 2 then n2 = reverse(sort(cvtime[0,*,m]))


                  print, format='("                 ",A9, 2x,f5.1, 1x,f5.1, 2x,f5.1, 1x,f5.1, 3x, f5.1, 3x, f7.1,7x, A)', srcname[n2[n]], cvtime[*, n2[n], m], max_slot[n2[n], m], wei[n2[n]], comment[n2[n]]
 	       Endfor

	Endfor

Endif

	printf, lun, 'The best  '+strtrim(string(npop),1)+'  POPs are : ', strtrim(name_slot+namefix, 1)
	Printf,lun, format='("The observation time thredshold for each avaliable source is : ", f4.2, " hours")', obst
 
        If keyword_set(daylightsav) eq 1 then $
        Printf, lun, format='("Using Daylight-Saving Time!")'
	printf, lun, format='("Local sunset time : ", f4.1, "  sunrise time : ", f4.1)', sunset, sunrise 
	printf, lun, format='("Observation starts ", f4.1, " hour after sunset, ends ", f4.1, " hour before sunrise")', Starttime,Endtime

	If optimize eq 1 then $
	Printf, lun, 'Optimization: optimizes the number of avaliable sources first, and then weighted observation time of sources.' else if optimize eq 2 then $
	printf, lun, 'Optimization: optimizes the observation time of sources according to their weights.'
        printf, lun, 'Obs. Slot 2 non-zero means there are two avaliable observation time slots'
	printf, lun, ''

	If sorting eq 1 then srt = '  sorted by duration' else $
	If sorting eq 2 then srt = '  sorted by starting time'
           printf, lun,format='("Local observation time for these POPs:  (in Hours,", A30, " )")', srt
        printf,lun, ''
        printf,lun, ''

        If n_elements(tele) eq 2 then begin
	        printf, lun,'                            Obs. Slot 1   Obs. Slot 2'
	        printf, lun,'                    Source   Start  End   Start  End   Duration   Weight    V     H     K      Comment'
        endif else begin
                printf, lun,'                            Obs. Slot 1  ' 
                printf, lun,'                    Source   Start  End   Duration  Weight    V      H      K    Comment'
        endelse


        For m = 0, npop-1 do begin
                printf, lun, ' POP: ', strtrim(name_slot[m]+namefix, 1)
                For n= 0, n_elements(src[0,*])-1 do begin 

			if sorting eq 1 then n2 = reverse(sort(max_slot[*,m]))
			if sorting eq 2 then begin        ; Sort by Starting Time
				tmp= cvtime[0,*,m]
				tmpidx1 = where(tmp le 10. and tmp gt 0)
				tmpidx2 = where( (tmp eq 0.) and (cvtime[1,*,m] eq 0.0 ))
				if tmpidx1[0] ne -1 then tmp[tmpidx1] = tmp[tmpidx1] + 24.00001
				if tmpidx2[0] ne -1 then tmp[tmpidx2] = tmp[tmpidx2] +100.	
                                tmp = round(tmp*10)/10.   ; trucate after one decimal digit
				n2 = sort(tmp)
                                
                                n3 = uniq(tmp(n2))        ; Now sort according to the Ending Time
                                tmp2 = cvtime[1,*,m]                
                                tmp2idx = where(tmp2 le 10. and tmp gt 0)
                                if tmp2idx[0] ne -1 then tmp2(tmp2idx) = tmp2(tmp2idx)+ 24.

                                n2_new = 0
              
                                for lp10 =0, n_elements(n3)-1 do begin
                                    n4 = where(tmp(n2) eq tmp(n2[n3[lp10]]))
                                    n5 = sort(tmp2(n2[n4]))
                                    n2_new = [n2_new, n2[n4[n5]]]
                                endfor
                               n2 = n2_new[1:*]  
			 endif

                        if n_elements(tele) eq 2 then begin
                           printf, lun, format='("                 ", A9, 2x,f5.1, 1x,f5.1, 2x,f5.1, 1x,f5.1, 3x, f5.1, 3x, f7.1,5x, A)',$
                                 srcname[n2[n]], cvtime[*, n2[n], m], max_slot[n2[n], m], wei[n2[n]], comment[n2[n]]
                        endif else begin
                           printf, lun, format='("                 ", A9, 2x,f5.1, 1x,f5.1, 3x, f4.1, 4x, f5.1,5x, A)',$
                                 srcname[n2[n]], cvtime[0:1, n2[n], m], max_slot[n2[n], m], wei[n2[n]], comment[n2[n]]
                        endelse
		Endfor
        Endfor

if n_elements(gid) ne 0 then widget_control, gid, /append, set_value=['---> The output is in file:  '+output, 'DONE!', '']

free_lun,lun
end
