;FUNCTION :  uvtrack
;
;Purpose: Calculate the uv track for one/several baseline(s) 
;           (xyz, east-north-up), with given source (ra, dec),
;           given array (lon, lat) and given time series (mjd or ha)
;
;Usage:    uv= uvtrack(xyz,ra,dec,lat,lon,mjd=mjd, [ha=ha])
;          use either mjd or ha. If hour angle is given, use ha instead of mjd  
;
;Input:    ra & dec:  the R.A. & Dec. of the source in RADIANS
;          lat &lon:  the position of the array in DEGREE
;          xyz     :  the (x,y,z) value of baseline(s) $
;                            in the cooradinate with X-axis to the East, $
;                            Y to the North, and Z is up, in METERS
;                     e.g., xyz=[[1,1,1],[2,2,2],[3,3,3]] for 3 baselines, $
;                            or [1,1,1] for 1 baseline
;          mjd     :  the time series of the observation in MJD (Modified Julian date)
;       or ha      :  the hour angle of the source in HOURS
;                     use either mjd or ha 
;
;Return:   uv={ u:u, v:v, w:w, mjd:mjd, ha:ha }
;          the uv values (in Meters) for the 'k'th
;          baseline are u[k]=uv.u[k,*], v[k]=uv.v[k,*] & w[k]=uv.w[k,*]
;
;Note:     If Lamda is not given, then lamda=1.0 
;          Need to call function 'arrcal' and 'zenpos'
;
;Ming Zhao 05/23/04
;modified, 06/30/04
;          01/24/05, refined the value of pi
;          03/04/05, modified to make the U,V in Meters, according to OI-FITS file format.
;          07/12/05, modified, don't use LAMBDA any more. So the output has the same unit as input U,V.

FUNCTION arrcal,ra,dec,h0,i0,arr0


; calculate  u,v,w in the projected uv plane (assume lamda=1 here) $
;      for a given time (h0) from the array coordinate (arr0=[x,y,z])
 
a=[[-sin(ra)         ,  cos(ra),          0       ],$
   [-cos(ra)*sin(dec), -sin(ra)*sin(dec), cos(dec)],$
   [ cos(ra)*cos(dec),  sin(ra)*cos(dec), sin(dec)]]

b=[[-sin(h0), -sin(i0)*cos(h0), cos(i0)*cos(h0)],$
   [ cos(h0), -sin(i0)*sin(h0), cos(i0)*sin(h0)],$
   [ 0.0   ,   cos(i0)        , sin(i0)        ]]

arr1=a##b##arr0
return,arr1
end
; -------------------------------------



FUNCTION uvtrack,xyz0,ra,dec,lat,lon,mjd=mjd,ha=ha


pi = !dpi


  ;Calculate the uv track for the baselines (xyz) 

;if (n_elements(lamda) eq 0) then begin
;lamda=1.0
;print,'LAMDA is not given in UVTRACK, default lamda=1.0'
;endif

xyz=transpose(xyz0)


  ; Get the zenith RA and Dec in RADIANS for a given JD

if n_elements(mjd) ne 0 then begin
   COMMON SITE, la,lng,tzone ; tzone is useless in zenpos for Julian date 
	la=lat
	lng=lon
	tzone=0.0
	jd=mjd+2400000.5d
	zenpos, jd,zra,i            ; Get the zenith RA (zra)  and Dec (i) 
	ha=(zra-ra)
	   z = where(ha gt pi,count)
	   if (count ne 0) then ha[z] = ha[z] - 2.0*pi
	   z = where(ha le -pi,count)
	   if (count ne 0) then ha[z] = ha[z] + 2.0*pi
	ha=ha/pi *12.0              ; Convert Pi to hours 

endif else $
if n_elements(ha) ne 0 then begin

  ; IF ha is given then use ha instead of mjd, otherwise calculate ha

	print, 'MJD is not given, use given HA instead'
	lha=ha/12d0 * !dpi
	zra=ra+ lha                ; zra is the LST 
	mjd=0.0
endif


n=n_elements(zra)      ; # of points in each uv track
k=n_elements(xyz)/3    ; # of baselines
arr=make_array(k,n,3,/double)


for j=0,k-1 do begin
	for m=0, n-1 do begin
  	    arr[j,m,*]=arrcal(ra, dec, zra[m],lat/180.d*pi,xyz[j,*])
        endfor
endfor 

;arr=arr/lamda          ; convert to the unit of wavelength

uv={u:arr[*,*,0], v:arr[*,*,1], w:arr[*,*,2], mjd:mjd, ha:ha}
; u[k]=uv.u[k,*] & v[k]=uv.v[k,*] & w[k]=uv.w[k,*], etc.

return, uv
end


