PRO gc_sim4, output=output, psf=psf, field_width=field_width, field_height=field_height, $
	bgrd_width=bgrd_width, bgrd_height=bgrd_height, rc=rc, r_min=r_min, $
	r_max=r_max, r_res=r_res, brightest=brightest, dimmest=dimmest, $
	resolution=resolution, n_trials=n_trials, deblend=deblend, norm_range=norm_range, $
	ndet_inRange=ndet_inRange, low=low, high=high, beta=beta, noise=noise, $
	gc3=gc3, alpha=alpha

;------------------------------------------------------------------
;+
; NAME:
;	gc_sim4
;
; PURPOSE:
;	In the galactic center, there is a large population of 
;	undetected stars that we do not very much about.
;	We can, however, make a good guess at the stellar
;	density and at the luminosity function of these stars.
;	This program uses a best-guess at the stellar distribution
;	at the center of the galaxy, then runs starfinder on it
;	to determine how many stars are detected at particular
;	magnitudes and radii, i.e. it estimates the completeness
;	at each magnitude and radial distance from SgrA*. This
;	completeness can be used to create simulations of the 
;	detected and undetected populations separately, used cheifly
;	in gc_sim3, where the astrometric and photometric errors
;	for stars going over the undetected background are determined.
;
;	A call to gc_sim3 can be used at the end of the program to
;	have gc_sim3 run immediately after gc_sim4.
;
;	A call to gc4_read is made automatically in gc_sim3 to analyze
;	and organize the data as necesscary, though if desired,
;	it may be called at the end of gc_sim4 to visualize the 
;	completeness
;
;
; INPUTS:
;	output: the name of the file that you want the completeness
;		arrays written in. 'gc_sim4.txt' by default.
;
;	psf: the location of the psf file that you want to use
;		to estimate the completeness with.
;
;	field_width and field_height: the sizes of the simulation
;		field (pix)
;
;	bgrd_width and bgrd_height: the size of the area that the 
;		simulated stars can appear in. Note that this should
;		be TWICE the largest distance from SgrA* that you 
;		want to get the completeness for, but either the same
;		size as or smaller than the field. It is the size of 
;		the field by default.
;
;	rc: the core radius to the spatial distribution. It is 30.0pix
;		by default, as that has been found to produce the 
;		best fit, though it may be different depending on
;		the data that you extrapolate from or the size of
;		the field that you are simulating.
;	r_min and r_max: the lower and upper limits, respectively,
;		of where (in pixels from SgrA*) that you want to get
;		the completeness for. It is 5 pixels smaller than the
;		half-width of the background by default, so as to still
;		get an accurate error for the edges, though it can
;		be adjusted, so long as it is smaller than 1/2 of the 
;		background size	.
;	r_res: the resolution of the radial bins holding the completeness
;		curves. It is 1/10 of the r_min-r_max range by default,
;		i.e. ten curves are made.
;	brightest and dimmest: the brightest and dimmest magnitude stars
;		that you want in your simulation. This should include
;		the entire distribution, down to whatever magnitude
;		stars are no longer bright enough to have influence
;		on any detected stars. These are 9.5 mag and 21.0 mag
;		by default.
;	resolution: the magnitude resolution that you want to have in
;		your completeness arrays. 0.25 mag by default.
;	n_trials: the number of times to resimulate the field so that
;		the completeness can be as smooth as computation power
;		and time allow. 200 by default.
;	/deblend: set this keyword to turn deblending on to detect
;		nearby stars. This carries the side effect of false
;		star detections.
;	norm_range, ndet_inRange, low, high, beta: the normalization
;		parameters for the spatial and magnitude distributions.
;		norm_range is the distance from SgrA* that your normalization
;		takes stars out to, low is the brightest magnitude in that
;		range, high is the dimmest, and beta is the slope of the
;		power law that the luminosity function is drawn from.
;
;	/noise: set this keyword to use noise in the simulation
;
;	/gc3: set this keyword if you want to run gc_sim3 immediately
;		after running gc_sim4, using the same parameters.
;
;	alpha: the slope of the radial number density distribution. -1.45 by default
;	
;
; MODIFICATION HISTORY:
;	08/22/2006 - Original version (James Dunn)
;	08/25/2006 - Added spatial distribution power law (Dunn)
;	09/05/2006 - Abel transformed and allowed for extrapolation
;		     of spatial dist (Dunn)
;	09/13/2006 - Completeness algorithm created
;-
;-------------------------------------------------------------

;--------------------------PARAMETERS------------------------------

if not keyword_set(psf) then begin                        
	;makes the default psf the following psf                          
	psf = readfits('/u/jkdunn/originals/mag06maylgs1_kp_psf.fits')
endif else begin
	psf1=psf
	psf = readfits(psf1)
endelse	

if not keyword_set(field_width) then field_width = 400.0
if not keyword_set(field_height) then field_height = 400.0
if not keyword_set(bgrd_width) then bgrd_width = field_width
if not keyword_set(bgrd_height) then bgrd_height = field_height

if not keyword_set(rc) then rc = 46.0 ;the core radius, used in the radial density function
if not keyword_set(alpha) then alpha = -1.45 ;slope of the radial distribution

if not keyword_set(r_min) then r_min = 0.0
if not keyword_set(r_max) then r_max = bgrd_width/2 - 5.0
if not keyword_set(r_res) then r_res = (r_max-r_min)/10.0 ;radial width of the light curves
r_range = r_max-r_min


if not keyword_set(brightest) then brightest = 9.5 ;brightest star in background (mag)
if not keyword_set(dimmest) then dimmest = 21.0 ;dimmest star in background (mag)
if not keyword_set(resolution) then resolution = 0.25 ;detected magnitude resolution

if not keyword_set(deblend) then deblend = 0 
if keyword_set(deblend) then deblend = 1

high_count = 27000000*10^((9.83-15.0)/2.5) ;photon count on bright
					   ;star, used for noise
					      
if not keyword_set(n_trials) then n_trials = 200 ;times to run through	





;-----------PSF ALIGNMENT AND RESIZE------------
full_psf = fltarr(field_width, field_height)
psf_size = sqrt(n_elements(psf))

if psf_size gt field_width OR psf_size gt field_height then begin
	print, 'PSF is larger than size of star field. Trim your psf or use a larger field.'
        retall
endif

for i=0,psf_size-1 do for j=0,psf_size-1 do full_psf[i,j]+=psf[i,j]






	;--------------ORIGIN ALIGNMENT AND SCALING-----------

	;calculates noise
	gauss_noise_std, full_psf, mode, sigma, patch=3, nterms_fit=3
	

	;sets the parameters for starfinder's astrometry/photometry
	threshold = 3. ;detection threshold (in noise stdev units)
	min_correlation = 0.8 ;minimum correlation (usually 0.8)
	correl_mag = 2
	psf_fwhm = fwhm(psf) ;used to detect stars
	back_box = 9.0 * psf_fwhm ;size for background estimation (usu. 9 psf_fwhm)
	niter = 2 ;number of refitting iterations (usu. 2)


	;calls starfinder with the above parameters
	starfinder, full_psf, psf, $
	BACK_BOX = back_box, $
	threshold, REL_THRESHOLD = 1, /PRE_SMOOTH, $
	NOISE_STD = sigma, min_correlation, $
	CORREL_MAG = correl_mag, $
	
	DEBLEND = deblend, $ ;this is often omitted because it can lead to false detections
	N_ITER = niter, $
	x_psf, y_psf, flux_psf, sx, sy, sf, correlation

	;aligns the center of the original star to the origin
	move = [-x_psf, -y_psf]
	full_psf = shift_interp(full_psf, move)





;-------DISTRIBUTIONS--------
;we use a power laws with some initial conditions to get our 
;luminosity (magnitude) distribution function and our stellar
;density distribution function



;----MAGNITUDE----


if not keyword_set(norm_range) then norm_range = 500.0 
range = norm_range   

if not keyword_set(ndet_inRange) then $    
	ndet_inRange = 217 ;the number of detected stars in the mag range
		  ;used to normalize the power law and within the
		  ;desired size of the background. This has an 
		  ;ENORMOUS effect on the distribution, including
		  ;the total number of background stars, so be 
		  ;sure that it is accurate. 
if not keyword_set(low) then $		  
	low = 9.5 ;the low end of the range used for normalization
if not keyword_set(high) then $
	high = 15.0 ;the high end ""
if not keyword_set(beta) then $
	beta = 0.21 ;the slope of the magnitude power law function
	   	    ;BE SURE THIS MATCHES WHAT YOU USE IN ten.pro
	    
norm_mag = 1.0/QROMB('ten',brightest, dimmest) ;the normalization constant for the luminosity function	    
	
a = ndet_inRange/QROMB('ten', low, high) ;integration constant
print, 'a is', a

ns = a/norm_mag ;total number of stars that should be present
		;in the smaller extrapolation field
		
print, 'first ns is', ns
	

;-----SPATIAL-----	
ds_res = 0.1 ;the bin width of the numerically calculated distribution
e0 = spatial(0.0, range, ds_res, NS=ns, RC=rc, alpha=alpha) ;the output here is the true 
					 ;distribution function	

print, 'e0 is', e0	


ns = spatial(0.0, bgrd_width/2, ds_res, E0=e0, RC=rc, alpha=alpha) ;ns is now the true number
					       ;of stars in the field
										       

ds = spatial(0.0, bgrd_width/2, ds_res, RC=rc, alpha=alpha) ;this will be the normalized,
					;cumulative function

	
	
	
;-----LIGHT CURVE PARAMETERS-----	
struct = {n:fltarr((dimmest-brightest)/resolution), $
	mag:indgen(((dimmest-brightest)/resolution))*resolution+brightest, $
	start:fltarr((dimmest-brightest)/resolution)}
radius = replicate(struct, r_range/r_res)



	;------BEGIN TRIALS LOOP--------
	for a=0, n_trials-1 do begin
	print, 'TRIAL # ', a+1, ' OUT OF ', n_trials





;--------SPATIAL DISTRIBUTION RANDOMIZATION-------

;draw directly from power law profile with slope = alpha


d = fltarr(ns)


print, 'RANDOMIZING STAR POSITIONS'
rnd = randomu(seed, ns)
for y=0,ns-1 do begin
	for e=1,n_elements(ds)-1 do begin
		if rnd[y] gt ds[e-1] AND rnd[y] lt ds[e] then $
			d[y]=e*ds_res
	endfor
endfor	
print, 'DONE'


;when the integral was binned, it allowed only discrete separations
;This lets the actual separations distribute randomly about those 
;discrete values
d += (randomu(seed, ns)-0.5)*ds_res


		

;---------SPATIAL HISTOGRAM----------
sbin_size = 5.0
sbins = fltarr((bgrd_width/2)/sbin_size)

for k=0, n_elements(d)-1 do begin
for i=0,((bgrd_width/2)/sbin_size)-1 do begin
	if d[k] gt (i*sbin_size) then begin
	if d[k] lt (i+1)*sbin_size then begin
		sbins[i]++
	endif
	endif
endfor
endfor

;this makes the plot a radial density plot
for i=0, ((bgrd_width/2)/sbin_size)-1 do sbins[i]/=!pi*(((i+1)*sbin_size)^2-(i*sbin_size)^2)


;holds the x values for the histogram bins
sxgraph=indgen((bgrd_width/2)/sbin_size)*sbin_size+(sbin_size/2)



window =1 & plot, sxgraph, sbins, title='radial distribution histogram', $
	xtitle='distance from SgrA* (pix)', ytitle='stars/pixel^2'


set_plot, 'ps'
device, filename='/u/jkdunn/gc_sim_rad.ps'
plot, sxgraph, sbins, title='radial distribution histogram', $
	xtitle='distance from SgrA* (pix)', ytitle='stars/pixel^2'
device, /close_file

openw, 5, 'gc_sim_rad.txt'
for i=0, n_elements(sxgraph)-1 do printf,5,sxgraph[i],sbins[i]
close, 5






;--------MAG DISTRIBUTION RANDOMIZATION------

mag = (1/beta)*alog10((randomu(seed, ns)*beta*alog(10.0)/norm_mag)+10^(beta*brightest))



;------MAG DISTRIBUTION HISTOGRAM-----
mbin_size = 0.25


for k=0, n_elements(mag)-1 do begin
for l=0, r_range/r_res-1 do begin

	if d[k] gt l*r_res+r_min AND d[k] lt (l+1)*r_res+r_min then begin

for i=0,((dimmest-brightest)/mbin_size)-1 do begin
	if mag[k] gt brightest+(i*mbin_size) $
	AND mag[k] lt brightest+((i+1)*mbin_size) then begin
		radius[l].start[i]++
	endif
endfor	
	endif

endfor
endfor

mxgraph=indgen((max(mag)/mbin_size)+1)*mbin_size




;----------BACKGROUND STAR INITIATION---------

fluxes = 10^((15.0-mag)/2.5)
theta = randomu(seed, ns)*2*!pi

background = fltarr(field_width, field_height)


print, 'INITIATING BACKGROUND FIELD...'
for i=0,ns-1 do begin
	star = full_psf*fluxes[i]	
	move = [d[i]*cos(theta[i])+(field_width/2), d[i]*sin(theta[i])+(field_height/2)]
	star = shift_interp(star, move)
	background += star
endfor
print, 'DONE'



	field = background
	
	
	
	if keyword_set(noise) then begin
	;-------NOISE---------------
	;noise is regenerated for each epoch so as to eliminate dependencies
	;and follow the moving star
	
	
	peak = max(full_psf)
	neg = where(high_count*(field/max(field)) lt 0 OR  $
			high_count*(field/max(field)) eq 0)
	;occasionally we get a point where the field is zero,
	;and in this case the noise is casted as infinite. This
	;makes the noise zero at those points	
	
	noise = (field / sqrt(high_count*(field/peak)) ) $
		*(randomu(seed,field_width,field_height,/normal)+1)

	if total(neg) gt 0 then noise[neg]=0.0
	
	field += noise	
	
	endif
	
	
	
	fits_write, '/u/jkdunn/simages/gc_sim4.fits', field
	

	
;-------------STAR DETECTION------------
	
	;calculates noise
	gauss_noise_std, field, mode, sigma, patch=3, nterms_fit=3
	

	;sets the parameters for starfinder's astrometry/photometry
	threshold = 3. ;detection threshold (in noise stdev units)
	min_correlation = 0.8 ;minimum correlation (usually 0.8)
	correl_mag = 2
	psf_fwhm = fwhm(psf) ;used to detect stars
	back_box = 9.0 * psf_fwhm ;size for background estimation (usu. 9 psf_fwhm)
	niter = 2 ;number of refitting iterations (usu. 2)


	;calls starfinder with the above parameters
	starfinder, field, psf, $
	BACK_BOX = back_box, $
	threshold, REL_THRESHOLD = 1, /PRE_SMOOTH, $
	NOISE_STD = sigma, min_correlation, $
	CORREL_MAG = correl_mag, $
	
	DEBLEND = deblend, $ ;this is often omitted because it can lead to false detections
	N_ITER = niter, $
	x_det, y_det, f_det, sx, sy, sf, correlation
	
	
	
	;retranslates the x and y detections into distance from sgrA*
	x_det -= field_width/2
	y_det -= field_height/2

	d_det = sqrt(x_det^2+y_det^2)
	m_det = 15.0 + 2.5*alog10(1.0/f_det)
	

	
	;------BINNING-------
	for i=0,n_elements(f_det)-1 do begin ;go thru detected stars
	for j=0,r_range/r_res-1 do begin ;go through radii
		
		if d_det[i] gt j*r_res+r_min AND d_det[i] lt (j+1)*r_res+r_min then begin
			;if we match the radius, j, then bin the mag
			
	for k=0,(dimmest-brightest)/resolution-1 do begin ;go through mags
		if m_det[i] gt k*resolution+brightest AND m_det[i] lt (k+1)*resolution+brightest then begin
			radius[j].n[k]++
		endif	
	endfor
	
		endif
	
	endfor
	endfor	
	
	
		
	
	;just gives a sort of counter for the program so that there is
	;some indicator of how complete the run is
	print, 'trial ', a+1, ' of ', n_trials
	
		;prints an image of the field
		set_plot, 'ps'
		device, filename='/u/jkdunn/gc_sim_field.ps'
		tvscl, field 
		device, /close_file	
		
		
		
endfor





;------DATA RECORDING--------


if not keyword_set(output) then output='gc_sim4.txt'
openw,1, output

for i=0,r_range/r_res-1 do begin
for j=0,(dimmest-brightest)/resolution-1 do begin

	printf,1, i*r_res+r_min, (i+1)*r_res+r_min, radius[i].mag[j]+resolution/2, radius[i].n[j], radius[i].start[j]
	
endfor
printf,1, ' '	
endfor	

close,1


if keyword_set(gc3) then begin

gc_sim3, gc4_output=output, psf=psf, field_width=field_width, $
field_height=field_height, bgrd_width=bgrd_width, bgrd_height=bgrd_height, $
rc=rc, r_min=r_min, r_max=r_max, brightest=brightest, dimmest=dimmest, $
deblend=deblend, norm_range=norm_range, alpha=alpha, $
ndet_inRange=ndet_inRange, low=low, high=high, beta=beta, noise=noise

endif


END

